==> hlog-0.16-rc1.tar.gz: hlog-0.16-rc1/common/qso_line.c <==
/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/error.h>
#include <jeffpc/val.h>
#include <jeffpc/sexpr.h>
#include <jeffpc/cstr.h>

#include <hlog/qso.h>
#include <hlog/qso_line.h>
#include <hlog/bands.h>
#include <hlog/util.h>

static void fmt_date(struct qso *qso, char *buf, size_t bufsize);
static void fmt_time_hms(struct qso *qso, char *buf, size_t bufsize);
static void fmt_station_call(struct qso *qso, char *buf, size_t bufsize);
static void fmt_freq_mhz(struct qso *qso, char *buf, size_t bufsize);
static void fmt_band(struct qso *qso, char *buf, size_t bufsize);
static void fmt_mode(struct qso *qso, char *buf, size_t bufsize);
static void fmt_rsts(struct qso *qso, char *buf, size_t bufsize);
static void fmt_rstr(struct qso *qso, char *buf, size_t bufsize);
static void fmt_tx_power(struct qso *qso, char *buf, size_t bufsize);
static void fmt_grid(struct qso *qso, char *buf, size_t bufsize);
static void fmt_seqr(struct qso *qso, char *buf, size_t bufsize);
static void fmt_seqs(struct qso *qso, char *buf, size_t bufsize);
static void fmt_county(struct qso *qso, char *buf, size_t bufsize);
static void fmt_state(struct qso *qso, char *buf, size_t bufsize);
static void fmt_country(struct qso *qso, char *buf, size_t bufsize);

static const struct printer {
	const char *name;
	size_t width;
	void (*fmt)(struct qso *, char *, size_t);
} printers[] = {
	{ "date",         10, fmt_date, },
	{ "time-hm",       5, fmt_time_hms, },
	{ "time-hms",      8, fmt_time_hms, },
	{ "station-call", 10, fmt_station_call, },
	{ "freq-mhz",     10, fmt_freq_mhz, },
	{ "band",          5, fmt_band, },
	{ "mode",          6, fmt_mode, },
	{ "rsts",          3, fmt_rsts, },
	{ "rstr",          3, fmt_rstr, },
	{ "tx-power",      5, fmt_tx_power, },
	{ "grid",          8, fmt_grid, },
	{ "grid8",         8, fmt_grid, }, /* convenience alias for 'grid' */
	{ "grid6",         6, fmt_grid, },
	{ "grid4",         4, fmt_grid, },
	{ "seqr",          4, fmt_seqr, },
	{ "seqs",          4, fmt_seqs, },
	{ "county",       10, fmt_county, },
	{ "state",         2, fmt_state, },
	{ "country",      10, fmt_country, },
};

/* large enough to print just about anything even repeatedly */
static const struct printer *line_printers[2 * ARRAY_LEN(printers)];

static inline void copy_str(char *buf, size_t bufsize, struct val *str)
{
	if (str)
		strcpy_safe(buf, val_cstr(str), bufsize);
}

static inline void copy_int(char *buf, size_t bufsize, struct val *val,
			    int width)
{
	if (val)
		snprintf(buf, bufsize, "%*"PRIu64, width, val->i);
}

static void fmt_date(struct qso *qso, char *buf, size_t bufsize)
{
	time_snprint_date(buf, bufsize, &qso->start);
}

static void fmt_time_hms(struct qso *qso, char *buf, size_t bufsize)
{
	time_snprint_time(buf, bufsize, &qso->start);
}

static void fmt_station_call(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.station_call);
}

static void fmt_freq_mhz(struct qso *qso, char *buf, size_t bufsize)
{
	pretty_print_freq(buf, bufsize, qso->tx.freq);
}

static void fmt_band(struct qso *qso, char *buf, size_t bufsize)
{
	const char *band;

	if (!qso->tx.freq)
		return; /* no freq => no band */

	band = bands_lookup_name_by_freq(qso->tx.freq->i);
	if (!band)
		return; /* freq outside all ham bands */

	strcpy_safe(buf, band, bufsize);
}

static void fmt_mode(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->tx.mode);
}

static void fmt_rsts(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->tx.rst);
}

static void fmt_rstr(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.rst);
}

static void fmt_tx_power(struct qso *qso, char *buf, size_t bufsize)
{
	pretty_print_power(buf, bufsize, qso->tx.power);
}

static void fmt_grid(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.grid);
}

static void fmt_seqr(struct qso *qso, char *buf, size_t bufsize)
{
	copy_int(buf, bufsize, qso->rx.seq, 3);
}

static void fmt_seqs(struct qso *qso, char *buf, size_t bufsize)
{
	copy_int(buf, bufsize, qso->tx.seq, 3);
}

static void fmt_county(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.county);
}

static void fmt_state(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.state);
}

static void fmt_country(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.country);
}

static const struct printer *find_printer(const char *name)
{
	size_t i;

	for (i = 0; i < ARRAY_LEN(printers); i++) {
		if (strcasecmp(printers[i].name, name))
			continue;

		return &printers[i];
	}

	return NULL;
}

/* initialize the default QSO line */
static void qso_line_init_defaults(void)
{
	line_printers[0] = find_printer("date");
	line_printers[1] = find_printer("time-hm");
	line_printers[2] = find_printer("station-call");
	line_printers[3] = find_printer("freq-mhz");
	line_printers[4] = find_printer("mode");
	line_printers[5] = find_printer("rsts");
	line_printers[6] = find_printer("rstr");
	line_printers[7] = find_printer("tx-power");
	line_printers[8] = find_printer("grid");
}

void qso_line_init(struct val *list)
{
	struct val *cur, *tmp;
	size_t i;

	i = 0;
	sexpr_for_each_noref(cur, tmp, list) {
		if (i >= ARRAY_LEN(line_printers))
			break;

		if (cur->type != VT_SYM)
			continue;

		line_printers[i] = find_printer(val_cstr(cur));
		i++;
	}

	val_putref(list);

	/* if the config has nothing, fall back to defaults */
	if (!i)
		qso_line_init_defaults();
}

void qso_line_fmt(struct qso *qso, char *buf, size_t bufsize)
{
	size_t i;

	ASSERT(bufsize > 0);

	buf[0] = '\0';

	for (i = 0; i < ARRAY_LEN(line_printers); i++) {
		const struct printer *prt = line_printers[i];

		if (!prt)
			continue;

		prt->fmt(qso, buf, bufsize);

		/*
		 * This field is bigger than the space left, it's already
		 * truncated.  Since there's no space left, we are done as
		 * well.
		 */
		if (bufsize <= prt->width)
			break;

		/* pad short fields */
		if (strlen(buf) < prt->width) {
			size_t thislen = strlen(buf);
			size_t padlen = prt->width - thislen;

			memset(buf + thislen, ' ', padlen);
		}

		/* truncate over-long fields */
		buf[prt->width] = '\0';

		buf += prt->width;
		bufsize -= prt->width;

		if (bufsize <= 2)
			break;

		buf[0] = ' ';
		buf[1] = '\0';
		buf++;
		bufsize--;
	}
}

==> hlog-0.20-rc1.tar.gz: hlog-0.20-rc1/common/qso_line.c <==
/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/error.h>
#include <jeffpc/val.h>
#include <jeffpc/cstr.h>

#include <hlog/qso.h>
#include <hlog/qso_line.h>
#include <hlog/bands.h>
#include <hlog/util.h>

static void fmt_date(struct qso *qso, char *buf, size_t bufsize);
static void fmt_time_hms(struct qso *qso, char *buf, size_t bufsize);
static void fmt_station_call(struct qso *qso, char *buf, size_t bufsize);
static void fmt_freq_mhz(struct qso *qso, char *buf, size_t bufsize);
static void fmt_band(struct qso *qso, char *buf, size_t bufsize);
static void fmt_mode(struct qso *qso, char *buf, size_t bufsize);
static void fmt_rsts(struct qso *qso, char *buf, size_t bufsize);
static void fmt_rstr(struct qso *qso, char *buf, size_t bufsize);
static void fmt_tx_power(struct qso *qso, char *buf, size_t bufsize);
static void fmt_grid(struct qso *qso, char *buf, size_t bufsize);
static void fmt_seqr(struct qso *qso, char *buf, size_t bufsize);
static void fmt_seqs(struct qso *qso, char *buf, size_t bufsize);
static void fmt_county(struct qso *qso, char *buf, size_t bufsize);
static void fmt_state(struct qso *qso, char *buf, size_t bufsize);
static void fmt_country(struct qso *qso, char *buf, size_t bufsize);

static const struct printer {
	const char *name;
	size_t width;
	void (*fmt)(struct qso *, char *, size_t);
} printers[] = {
	{ "date",         10, fmt_date, },
	{ "time-hm",       5, fmt_time_hms, },
	{ "time-hms",      8, fmt_time_hms, },
	{ "station-call", 10, fmt_station_call, },
	{ "freq-mhz",     10, fmt_freq_mhz, },
	{ "band",          5, fmt_band, },
	{ "mode",          6, fmt_mode, },
	{ "rsts",          3, fmt_rsts, },
	{ "rstr",          3, fmt_rstr, },
	{ "tx-power",      5, fmt_tx_power, },
	{ "grid",          8, fmt_grid, },
	{ "grid8",         8, fmt_grid, }, /* convenience alias for 'grid' */
	{ "grid6",         6, fmt_grid, },
	{ "grid4",         4, fmt_grid, },
	{ "seqr",          4, fmt_seqr, },
	{ "seqs",          4, fmt_seqs, },
	{ "county",       10, fmt_county, },
	{ "state",         2, fmt_state, },
	{ "country",      10, fmt_country, },
};

/* large enough to print just about anything even repeatedly */
static const struct printer *line_printers[2 * ARRAY_LEN(printers)];

/* iterative initialization index */
static size_t current_printer = ~0ul;

static inline void copy_str(char *buf, size_t bufsize, struct val *str)
{
	if (str)
		strcpy_safe(buf, val_cstr(str), bufsize);
}

static inline void copy_int(char *buf, size_t bufsize, struct val *val,
			    int width)
{
	if (val)
		snprintf(buf, bufsize, "%*"PRIu64, width, val->i);
}

static void fmt_date(struct qso *qso, char *buf, size_t bufsize)
{
	time_snprint_date(buf, bufsize, &qso->start);
}

static void fmt_time_hms(struct qso *qso, char *buf, size_t bufsize)
{
	time_snprint_time(buf, bufsize, &qso->start);
}

static void fmt_station_call(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.station_call);
}

static void fmt_freq_mhz(struct qso *qso, char *buf, size_t bufsize)
{
	pretty_print_freq(buf, bufsize, qso->tx.freq);
}

static void fmt_band(struct qso *qso, char *buf, size_t bufsize)
{
	const char *band;

	if (!qso->tx.freq)
		return; /* no freq => no band */

	band = bands_lookup_name_by_freq(qso->tx.freq->i);
	if (!band)
		return; /* freq outside all ham bands */

	strcpy_safe(buf, band, bufsize);
}

static void fmt_mode(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->tx.mode);
}

static void fmt_rsts(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->tx.rst);
}

static void fmt_rstr(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.rst);
}

static void fmt_tx_power(struct qso *qso, char *buf, size_t bufsize)
{
	pretty_print_power(buf, bufsize, qso->tx.power);
}

static void fmt_grid(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.grid);
}

static void fmt_seqr(struct qso *qso, char *buf, size_t bufsize)
{
	copy_int(buf, bufsize, qso->rx.seq, 3);
}

static void fmt_seqs(struct qso *qso, char *buf, size_t bufsize)
{
	copy_int(buf, bufsize, qso->tx.seq, 3);
}

static void fmt_county(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.county);
}

static void fmt_state(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.state);
}

static void fmt_country(struct qso *qso, char *buf, size_t bufsize)
{
	copy_str(buf, bufsize, qso->rx.country);
}

static const struct printer *find_printer(const char *name)
{
	size_t i;

	if (!name)
		return NULL;

	for (i = 0; i < ARRAY_LEN(printers); i++) {
		if (strcasecmp(printers[i].name, name))
			continue;

		return &printers[i];
	}

	return NULL;
}

/* initialize the default QSO line */
void qso_line_init_defaults(void)
{
	line_printers[0] = find_printer("date");
	line_printers[1] = find_printer("time-hm");
	line_printers[2] = find_printer("station-call");
	line_printers[3] = find_printer("freq-mhz");
	line_printers[4] = find_printer("mode");
	line_printers[5] = find_printer("rsts");
	line_printers[6] = find_printer("rstr");
	line_printers[7] = find_printer("tx-power");
	line_printers[8] = find_printer("grid");
}

void qso_line_init_start(void)
{
	ASSERT3U(current_printer, ==, ~0ul);

	current_printer = 0;
}

void qso_line_init_add(const char *name)
{
	const struct printer *p;

	ASSERT3U(current_printer, <, ARRAY_LEN(line_printers));

	p = find_printer(name);
	if (!p)
		return;

	line_printers[current_printer] = p;
	current_printer++;
}

void qso_line_init_done(void)
{
	size_t i;

	ASSERT3U(current_printer, <=, ARRAY_LEN(line_printers));

	for (i = current_printer; i < ARRAY_LEN(line_printers); i++)
		line_printers[i] = NULL;

	current_printer = ARRAY_LEN(line_printers);
}

void qso_line_fmt(struct qso *qso, char *buf, size_t bufsize)
{
	size_t i;

	ASSERT(bufsize > 0);

	buf[0] = '\0';

	for (i = 0; i < ARRAY_LEN(line_printers); i++) {
		const struct printer *prt = line_printers[i];

		if (!prt)
			continue;

		prt->fmt(qso, buf, bufsize);

		/*
		 * This field is bigger than the space left, it's already
		 * truncated.  Since there's no space left, we are done as
		 * well.
		 */
		if (bufsize <= prt->width)
			break;

		/* pad short fields */
		if (strlen(buf) < prt->width) {
			size_t thislen = strlen(buf);
			size_t padlen = prt->width - thislen;

			memset(buf + thislen, ' ', padlen);
		}

		/* truncate over-long fields */
		buf[prt->width] = '\0';

		buf += prt->width;
		bufsize -= prt->width;

		if (bufsize <= 2)
			break;

		buf[0] = ' ';
		buf[1] = '\0';
		buf++;
		bufsize--;
	}
}
